package main

import (
	"net/http"

	log "github.com/Sirupsen/logrus"
	"github.com/allegro/marathon-consul/config"
	service "github.com/allegro/marathon-consul/consul"
	"github.com/allegro/marathon-consul/marathon"
	"github.com/allegro/marathon-consul/metrics"
	"github.com/allegro/marathon-consul/sync"
	"github.com/allegro/marathon-consul/web"
)

var VERSION string

func main() {

	log.WithField("Version", VERSION).Info("Starting marathon-consul")

	config, err := config.New()
	if err != nil {
		log.Fatal(err.Error())
	}

	err = metrics.Init(config.Metrics)
	if err != nil {
		log.Fatal(err.Error())
	}

	service := service.New(config.Consul)
	remote, err := marathon.New(config.Marathon)
	if err != nil {
		log.Fatal(err.Error())
	}

	sync := sync.New(config.Sync, remote, service)
	go sync.StartSyncServicesJob()

	handler, stop := web.NewHandler(config.Web, remote, service)
	defer stop()

	// set up routes
	http.HandleFunc("/health", web.HealthHandler)
	http.HandleFunc("/events", handler)

	log.WithField("Port", config.Web.Listen).Info("Listening")
	log.Fatal(http.ListenAndServe(config.Web.Listen, nil))
}
